/*  Print screen image of form. 

*/

/* #define DEBUG */

#include <string.h>
#include <ctype.h>
#include "globals.h"

static char print_line[133], mask_line[133];
static int printer_line;

/* REM_UL --------------------

	Removes underlines from a string
*/
void rem_ul(str)
char *str;
{  char *p;

	for (p=str; *p; p++)
	  if (*p == '_') *p = ' ';
}

/* STRIP_TRAILING --------------------

	Removes trailing spaces from a string
*/
void strip_trailing(char *str)
{ int i;
  char *p;
	for (i = strlen(str), p = str+i-1; i>0; i--, p--)
	  if (*p == ' ') *p = '\0'; else break;
}

static void send_nl()
{
	fputs("\r\n",prn_file);
	printer_line++;
	if (form_len && pause_mode && (printer_line % form_len ==0))
	      { setcur(24,0);
	        wstr("\1Change Form and press RETURN.\2",1);
		putchar(7);	/* BELL */
		while (gtkey()!='\n') ;
		setcur(24,0);
		wstr(blanks,1);
	      }
}

/* PRINT_A_LINE --------------------

	Prints the current line, and clears
	the line buffers for the next line.

	Scans down the print list for all control constants to be printed
	on the current line, and sends them before the appropriate character
	in the print line.
*/

static void print_a_line(int printrow, struct field *top_of_line)
{ char *p;
  struct field *afield;
  int printcol=0;

#ifdef DEBUG
	printf("  Printing: %s\n",print_line);
#endif
	rem_ul(print_line);	/* Convert underlines in print line to spaces */

	p = print_line;
	afield = top_of_line;
	for (printcol=0; printcol < left_mgn; ++printcol)
		fputc(' ',prn_file);
	printcol = 0;
	while (afield != NULL && afield -> print_row == printrow)
	  { if (afield -> type != -1)		/* Find control sequence */
	      { afield = afield -> next_print;
		continue;			/* Loop until we find valid
						   ctrl seq on this line, or
						   field which goes on next line */
	      }
	    if (afield -> print_col == printcol)
	      {	fputs(afield -> mask,prn_file);	/* Send control sequence */
		afield = afield -> next_print;	/* and go to next ctrl seq */
	      }
	    else /* Printing before this control sequence */
	      {	fputc(*p++,prn_file);		/* Send one character */
		printcol++;			/* and advance logical carriage */
	      }
	  }
	strip_trailing(p);
	fputs(p,prn_file);	/* Send remainder of print line */

	if (ul_mode)
	  { for (p = mask_line; *p != '\0'; p++)
		if (*p != '_') *p = ' ';	/* Underlines & spaces only. */
	    strip_trailing(mask_line);
	    if (*mask_line)		/* Something in mask line? */
	      {	fputc('\r',prn_file);	/* Send CR */
		for (printcol=0; printcol < left_mgn; ++printcol)
			fputc(' ',prn_file);
		fputs(mask_line,prn_file);
	      }
	  }
	memset (print_line,' ',132); print_line[132] = '\0';
	strcpy(mask_line,print_line);
	send_nl();
}

/* --------------------
	PRINT_FORM
   --------------------

	Prints the form.  Interprets all the mode flags and does
	the appropriate action.
*/
int print_form()
{   struct field *afield, *top_of_line;
    int got_item, quote_this_field;
    int ix, x;
    char *p, *q;

    if (data_mode)			/* Data mode */
      {	afield = first_field;
	got_item = quote_this_field = 0;
	while (afield != NULL)
	  { if (afield -> work != NULL && !(afield -> non_printing))
		/* For each printing data field... */
	      {	if (got_item++)		/* Now sending 2nd field */
		  fputc(',',prn_file);	/* Separate with comma */

		quote_this_field = quote_mode && 
			(afield -> type < 2 || afield -> type == 6);
			/*  Text, Numeric, or Date get quoted */

		if (quote_this_field)
		  fputc(34,prn_file);	/* Begin field w/ quote */

		if (strcmp(afield -> mask, afield -> work))
			strcpy(print_line, afield -> work);
		else
			*print_line = '\0';	/* No data in this field */

		/* Convert underlines to spaces: */
		rem_ul(print_line);
		p = q = print_line;
		while (isspace(*q)) q++;
		do *p++ = *q; while (*q++);	/* Remove leading spaces */

		strip_trailing(print_line);
		fputs(print_line,prn_file);

		if (quote_this_field)
		  fputc(34,prn_file);
	      }
	    afield = afield -> next;	/* Go to next field */
	  }

	fputs("\r\n",prn_file);		/* Terminate record with Newline */

	fflush(prn_file);
	return(0);
      }

    /* Print all fields */

    fputc('\r',prn_file);		/* Send CR first */
    if (ferror(prn_file))		/* Error printing form? */
	return(1);

    for (x=0; x < top_mgn; ++x)		/* Send top margin */
      putc('\n',prn_file);

	printer_line = top_mgn;		/* Remember which line of form */
	memset (print_line,' ',132); print_line[132] = '\0';
	strcpy(mask_line,print_line);

    /* Print form from the print item linked list */
    afield = top_of_line = first_print_field;
    while (afield != NULL)
      {	if (afield -> print_row != printer_line-top_mgn)
						/* On to next line... */
	  { print_a_line(printer_line,top_of_line);
						/* Send current line */
	    for (ix=printer_line-top_mgn; ix<afield->print_row; ix++)
	      send_nl();			/* Skip to line w/ next field */
	    top_of_line = afield;		/* Here's start of next line */
	  }
	/* Now we're on the right line for this field -- insert it. */

	/* Handle background lines (SCREEN mode) */
	if (afield -> entry_line != NULL)
	    {	memcpy(print_line,afield->entry_line,
				strlen(afield->entry_line));
	    }

	/* Handle data fields */
	switch (afield -> type)
	 {case 0:	/* Constant Data (or background lines to ignore) */
		if (afield -> mask != NULL)
		    {	memcpy(&print_line[afield->print_col],afield->mask,
				strlen(afield->mask));
		    }
	  case -1:	/* Control Constants handled in print_line... */
		break;
	  default:	/* Data field */
		if (afield -> work != NULL && !(afield -> non_printing))
		  { if(strcmp(afield -> work, afield -> mask))
			/* Only print fields which are not identical to their masks
			   i.e., non-blank fields */
		      {	memcpy(&mask_line[afield->print_col],afield->mask,
				strlen(afield->mask));
			if (afield -> rightjust)
			  { char buf[80],*p;
			    int i, leftx;

			    leftx = afield -> print_col;
			    strcpy(buf,afield->work);
			    i=strlen(buf); p = buf+i-1;
			    while (i>0 && (*p == '_' || *p == ' '))
			      {*p-- = '\0'; i--; leftx++;}
			    if (i)
			      memcpy(&print_line[leftx],buf,strlen(buf));
			  }
			else
			  memcpy(&print_line[afield->print_col],afield->work,
				strlen(afield->work));
		      }
		  }
		break;
	 }
	afield = afield -> next_print;
      }
    print_a_line(printer_line,top_of_line);	/* Print last line */

    /*  End with ff. */
    if (form_feed)
      if (form_len ==0)		/* Printer form-feed */
	fputc('\f',prn_file);
      else			/* Simulate with linefeeds, to # of lines specified */
	while (printer_line < form_len)
	  send_nl();

    fflush(prn_file);
    return(0);
}
